# Copyright (C) 2009  Xyne
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


package Xyne::Common::Functions;
use warnings;
use strict;

use Cwd;
use Term::ANSIColor;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = do {
  no strict 'refs';
  grep exists &$_ && /[a-z]/, keys %{ __PACKAGE__ . '::'};
};

our $VERSION = '0.02';


######################################
########## Argument Parsing ##########
######################################

sub dejuxtapose_args
{
  my @dejux = ();
  foreach my $arg (@_)
  {
    if ($arg =~ m/^-([^-]\S+)$/)
    {
      push @dejux, map {"-$_"} split (//,$1);
    }
    else
    {
      push @dejux, $arg;
    }
  }
  return @dejux;
}

################################
########## Filesystem ##########
################################

sub find_all
{
  my @files = ();
  foreach my $path (@_)
  {
    $path = &strip_trailing_slash($path);
    if (-d $path)
    {
      opendir(my $dh,$path);
      foreach my $item (grep {m/[^\.]/} readdir($dh))
      {
        my $item_path = "$path/$item";
        if (-d $item_path)
        {
          push @files, &find_files($item_path);
        }
        push @files, $item_path;
      }
    }
    elsif (-f $path)
    {
      push @files, $path;
    }
  }
  return @files;
}

sub find_files
{
  my @files = ();
  foreach my $path (@_)
  {
    $path = &strip_trailing_slash($path);
    if (-d $path)
    {
      opendir(my $dh,$path);
      foreach my $item (grep {m/[^\.]/} readdir($dh))
      {
        my $item_path = "$path/$item";
        if (-f $item_path)
        {
          push @files, $item_path;
        }
        elsif (-d $item_path)
        {
          push @files, &find_files($item_path);
        }
      }
    }
    elsif (-f $path)
    {
      push @files, $path;
    }
  }
  return @files;
}

sub get_absolute_path
{
  my ($path) = @_;
  &strip_trailing_slash($path);
  my $f = substr($path,0,1);
  if ($f ne '/')
  {
    my $cwd = getcwd;
    if ($f eq '.' and substr($path,1,1) ne '.')
    {
      $path =~ s/^\./$cwd/;
    }
    else
    {
      $path = "$cwd/$path";
    }
    
  }
  while ($path =~ s/\/[^\/]*\/\.\.//){next;}
  return $path;
}

sub mkdir_p
{
  my ($path) = @_;
  my $start = '';
  foreach my $step (split /\/+/,$path)
  {
    next if not $step;
    $start .= '/' . $step;
    mkdir $start if (not -d $start);
    return 1 if not (-d $start);
  }
}

################################
########## Formatting ##########
################################

sub colorize
{
  my ($str,$color) = @_;
  return $str if (not (defined($color) and defined($str)));
  return color($color).$str.color('reset');
}

sub decolorize
{
  my ($str) = @_;
  $str =~ s/\x1B\[(?:\d;)?\d\d?m//g;
  return $str;
}

sub format_seconds($)
{
  my ($seconds,$minutes,$hours);
  $seconds = int(shift);
  return "$seconds s" if ($seconds < 60);
  ($minutes,$seconds) = &divmod($seconds,60);
  return sprintf("%d:%02d",$minutes,$seconds) if ($minutes < 60);
  ($hours,$minutes) = &divmod($minutes,60);
  return sprintf("%d:%02d:%02d",$hours,$minutes,$seconds);
}

sub format_bytes()
{
  my ($size,$type) = @_;
  $size = int($size);
  return $size.' B' if $type eq 'B';
  my $factor = ($type =~ m/i/) ? 0x400 : 1000;
  my $power;
  if (substr($type,0,1) eq 'T') { $power = 4; }
  elsif (substr($type,0,1) eq 'G') { $power = 3; }
  elsif (substr($type,0,1) eq 'M') { $power = 2; }
  elsif (substr($type,0,1) eq 'k') { $power = 1; }
  return sprintf("%.2f $type", $size / ($factor**$power)) if $power;
  
  my $sign = 1;
  if ($size < 0) {$sign = -1; $size *= -1; }
  return ($size*$sign).' B' if $size < $factor;
  
  my $divisor;
  my $unit;
  if ($size < ($divisor = $factor)*$factor) { $unit = 'kiB'; }
  elsif ($size < ($divisor = $divisor*$factor)*$factor) { $unit = 'MiB'; }
  elsif ($size < ($divisor = $divisor*$factor)*$factor) { $unit = 'GiB'; }
  elsif ($size < ($divisor = $divisor*$factor)*$factor) { $unit = 'TiB'; }
  $unit =~ s/i// if $factor == 1000;
  return sprintf("%.2f $unit", ($sign * $size) / $divisor);
}

sub strip_trailing_slash
{
  my ($str) = @_;
  return $str if $str eq '';
  return (substr($str,-1) eq '/') ? substr($str,0,-1) : $str;
}

sub wrap_line
{
  my ($width,$indent,@words) = @_;
  return '' if not ($width and defined($indent) and @words);
  my $text = '';
  my $line=$indent.shift(@words);
  foreach my $word (@words)
  {
    if (length($line) + 1 +length($word) <= $width)
    {
      $line .= ' '.$word;
    }
    else
    {
      $text .= $line."\n";
      $line = $indent.$word;
    }
  }
  $text .= $line;
  return $text."\n";
}

#############################
########## Generic ##########
#############################

sub confirm
{
  my ($question,$default_answer) = @_;
  print "$question ";
  if (defined($default_answer)) { print lc($default_answer) eq 'y' ? '[Y/n] ' : '[y/N] '; }
  else { print '[y/n] '; }
  my $answer = lc(<STDIN>); chomp $answer;
  $answer =  $default_answer if ($answer eq '');
  while ($answer ne 'y' and $answer ne 'n')
  {
    print "Please enter 'y' or 'n': ";
    $answer = lc(<STDIN>); chomp $answer;
  }
  return ($answer eq 'y') ? 1 : 0;
}

sub hash_map
{
  my ($hash,@args) = @_;
  return map { defined($hash->{$_}) ? $hash->{$_} : $_ } @args;
}

#################################
########## Mathematics ##########
#################################

sub divmod
{
  my ($dividend, $divisor) = @_;
  my $quotient = int($dividend/$divisor);
  my $remainder = $dividend % $divisor;
  return ($quotient,$remainder);
}


1;

__END__

=head1 NAME

Xyne::Common::Functions - Commonly used functions for Xyne's modules.

=head1 SYNOPSIS

  use Xyne::Common::Functions;

=head1 DESCRIPTION

This module keeps commonly used functions in one place to facilitate
maintenance.

=head2 EXPORT

None by default.

=head1 AUTHOR

Xyne <lt>xyne at archlinux dot ca<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2009  Xyne

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

=cut