# Copyright (C) 2009  Xyne
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


package Xyne::Common::Shell;
use strict;
use warnings;

use IPC::Open2;

our $VERSION = '0.01';

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self  = {};
  bless ($self, $class);
  
  # set the binary
  $self->{BINARY} = '/bin/bash';
  
  # set the "end of output" tag used internally to
  # when the shell output is done
  $self->{EOO_TAG} = '[END OF SHELL OUTPUT]:';
  
  # the callback will be used to display shell output
  # as it's read, if defined
  $self->{CALLBACK} = undef;
  
  # shell exit status for determining command sucess
  $self->{EXIT_STATUS} = 0;

  return $self;
}

sub DESTROY
{
  my ($self) = @_;
  $self->close_pipes;
}

sub close_pipes
{
  my ($self) = @_;
  close $self->{IN} if defined($self->{IN});
  waitpid ($self->{PID},0) if defined($self->{PID});
}

sub execute
{
  my ($self,@cmds) = @_;
  return $self->pipe_to_shell(@cmds);
}

sub get_binary
{
  my ($self) = @_;
  return $self->{BINARY};
}

sub get_callback
{
  my ($self) = @_;
  return $self->{CALLBACK};
}

sub get_exit_status
{
  my ($self) = @_;
  return $self->{EXIT_STATUS};
}

sub maybe_open_pipes
{
  my ($self) = @_;
  if ( not (defined($self->{IN}) or defined($self->{PID}) ) ) 
  {
    $self->close_pipes;
    $self->open_pipes;
  }
}

sub open_pipes
{
  my ($self) = @_;
  my $shell = $self->{BINARY};
  my $pid = open2(my $out, my $in, $shell)
  or die "error: failed to open pipes to $shell ($!)\n";
  $self->{PID} = $pid;
  $self->{IN} = $in;
  $self->{OUT} = $out;
}

sub pipe_to_shell
{
  $|++;
  my ($self,@cmds) = @_;
  $self->maybe_open_pipes;
  my $in = $self->{IN};
  my $out = $self->{OUT};
  my $eoo_tag = $self->{EOO_TAG};
  my @output;
  
  print $in "$_\n" foreach @cmds;
  # include the exit status
  print $in "echo \'$eoo_tag\' \$?\n";
  while (defined(my $line = <$out>))
  {
    if ($line =~ m/^\Q$eoo_tag\E (\d+)/)
    {
      $self->{EXIT_STATUS} = $1;
      last;
    }
    push @output,$line;
    $self->{CALLBACK}->($line) if defined( $self->{CALLBACK} );
  }
  return @output;
}

sub set_binary
{
  my ($self,$bin) = @_;
  $self->{BINARY} = $bin;
}

sub set_callback
{
  my ($self,$callback) = @_;
  $self->{CALLBACK} = $callback;
}

1;

__END__

=head1 NAME

Xyne::Common::Shell - A shell wrapper class.

=head1 SYNOPSIS

  use Xyne::Common::Shell;

=head1 DESCRIPTION

A simple shell wrapper that uses 2-way pipes to
provide shell access within an application while
avoiding the overhead of multiple shell invocation
and providing other features.

=head2 EXPORT

None by default.

=head1 AUTHOR

Xyne <lt>xyne at archlinux dot ca<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2009  Xyne

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

=cut